#
# import json
#
# import spacy
# import numpy as np
# from nltk.tokenize import sent_tokenize
# from nltk.corpus import stopwords
# class SemanticSimilarityTextChunker:
#     def __init__(self, query):
#         self.query = query
#         self.nlp = spacy.load("en_core_web_sm")
#         self.stop_words = set(stopwords.words('english'))
#
#     def preprocess_text(self, text):
#         doc = self.nlp(text.lower())
#         return [token.lemma_ for token in doc if token.text not in self.stop_words and token.is_alpha]
#
#     def get_query_keywords(self):
#         return self.preprocess_text(self.query)
#
#     def calculate_similarity(self, word1, word2):
#         return word1.similarity(word2)
#
#     def score_sentence(self, sentence, keywords):
#         preprocessed_sentence = self.preprocess_text(sentence)
#         doc_sentence = self.nlp(" ".join(preprocessed_sentence))
#         doc_keywords = self.nlp(" ".join(keywords))
#
#         if not doc_sentence.vector.any() or not doc_keywords.vector.any():
#             return 0
#
#         # Calculate the average similarity between each word in the sentence and the keywords
#         similarities = []
#         for token in doc_sentence:
#             if token.has_vector:
#                 keyword_similarities = [self.calculate_similarity(token, keyword) for keyword in doc_keywords if
#                                         keyword.has_vector]
#                 if keyword_similarities:
#                     similarities.append(max(keyword_similarities))
#
#         # Calculate the average similarity score
#         avg_similarity = np.mean(similarities) if similarities else 0
#
#         # Adjust the score based on the presence of exact matches
#         exact_match_bonus = sum(1 for word in preprocessed_sentence if word in keywords)
#         adjusted_score = (avg_similarity + 0.1 * exact_match_bonus) / (1 + 0.1 * len(keywords))
#
#         return adjusted_score
#
#     def chunk_text(self, text, chunk_size=3,top_chunks_in_article=3):
#         sentences = sent_tokenize(text)
#         keywords = self.get_query_keywords()
#
#         # Score each sentence
#         scored_sentences = [(sentence, self.score_sentence(sentence, keywords)) for sentence in sentences]
#
#         # Sort sentences by score in descending order
#         scored_sentences.sort(key=lambda x: x[1], reverse=True)
#
#         # Group top-scoring sentences into chunks
#         chunks = []
#         for i in range(0, min(len(scored_sentences), chunk_size * 3), chunk_size):
#             chunk = ' '.join([sent[0] for sent in scored_sentences[i:i + chunk_size]])
#             chunks.append(chunk)
#
#         return chunks[:top_chunks_in_article]  # Return top 3 chunks


# from .researcher_ai import ContextExtractor,MultiAgentQueryOrchestrator
# from .web_search import WebSearchAgent,FastHTMLParserV3,OptimizedMultiQuerySearcher,WebSearchArguments
# from .researcher_ai.main.search_provider.searcher import RealTimeGoogleSearchProvider
# from .researcher_ai.main.parse_url.html_parser import UrlTextParser
from .custom_ollama import CustomChatOllama
from .query_master.wordllama_embeds import WordLLamaRetriever
from .query_master.search_queries_generator import SearchQueryToNSubquery
from .query_master.query_refiner import QueryRefiner



import json
import re

from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from pydantic import BaseModel
from typing import List, Dict
import logging
import time

from testing.impl.events.test_event_processor import timestamp

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)


class Tweet(BaseModel):
    text: str
    timestamp: str


class TwitterContent(BaseModel):
    tweets: List[Tweet] = []
    page_source: str = ""


class TwitterScraper:
    def __init__(self, chromedriver_path="/usr/local/bin/chromedriver", headless=True):
        self.chromedriver_path = chromedriver_path
        self.headless = headless
        self.driver = None
        self.username = None
        self.password = None

    def _setup_driver(self):
        chrome_options = Options()
        if self.headless:
            chrome_options.add_argument("--headless")
        chrome_options.add_argument("--disable-gpu")
        chrome_options.add_argument("--no-sandbox")
        chrome_options.add_argument("--disable-dev-shm-usage")
        chrome_options.add_argument("--start-maximized")
        return webdriver.Chrome(self.chromedriver_path, options=chrome_options)

    def login_to_twitter(self, username, password):
        self.driver = self._setup_driver()
        self.driver.get("https://x.com/login")
        self.username = username
        self.password = password

        # Wait for the username field and enter the username
        username_field = WebDriverWait(self.driver, 10).until(
            EC.presence_of_element_located((By.NAME, "text"))
        )
        username_field.send_keys(username)
        self.driver.find_element(By.XPATH, "//span[text()='Next']").click()

        # Wait for the password field and enter the password
        password_field = WebDriverWait(self.driver, 10).until(
            EC.presence_of_element_located((By.NAME, "password"))
        )
        password_field.send_keys(password)
        self.driver.find_element(By.XPATH, "//span[text()='Log in']").click()

        # Wait for the home page to load
        WebDriverWait(self.driver, 10).until(
            EC.presence_of_element_located((By.XPATH, "//div[@data-testid='primaryColumn']"))
        )

    def get_home_page_content(self,num_tweets=2):
        if not self.driver:
            raise Exception("You must login first using login_to_twitter method")

        # Scroll down a few times to load more tweets
        page_content = ""

        # Scroll down a few times to load more tweets
        for _ in range(num_tweets):
            page_content += self.driver.page_source
            self.driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
            WebDriverWait(self.driver, 10).until(
                EC.presence_of_element_located((By.XPATH, "//article[@data-testid='tweet']"))
            )
        with open("home_page_tweets.txt","w") as f:
            f.write(f'{page_content}')

        tweets= self.get_tweets_from_text(self.username,page_content)

        return TwitterContent(
            tweets=tweets,
            page_source=page_content
        )

    def get_tweets_from_text(self,username,text):
        spans = re.findall('<span(.*?)>\s*(.*?)\s*</span>', text, re.DOTALL)
        spans = [_[1] for _ in spans]
        tweets = []
        isp = []
        start = False
        for span in spans:
            if span and span[0] == '@':
                start = True
                isp = []
            if start:
                if 'span' in span:
                    start = False
                    tweets.append("".join(isp))
                else:
                    if username not in span:
                        isp.append(span)
        return [Tweet(text = t,timestamp = "") for t in tweets]

    def get_user_tweets(self, username: str, num_tweets: int = 2):
        if not self.driver:
            raise Exception("You must login first using login_to_twitter method")

        self.driver.get(f"https://x.com/{username}")



        # # Wait for tweets to load
        # WebDriverWait(self.driver, 10).until(
        #     EC.presence_of_element_located((By.XPATH, f'{username}'))
        # )
        time.sleep(1)

        page_content = ""

        # Scroll down a few times to load more tweets
        for _ in range(num_tweets):
            page_content += self.driver.page_source
            self.driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
            WebDriverWait(self.driver, 10).until(
                EC.presence_of_element_located((By.XPATH, "//article[@data-testid='tweet']"))
            )

        tweets = self.get_tweets_from_text(username = username,
                                  text = page_content)

        return TwitterContent(
            tweets=tweets,
            page_source=self.driver.page_source
        )

    def parse_saved_html(self, content: str) -> List[str]:
        soup = BeautifulSoup(content, 'html.parser')

        tweet_spans = soup.find_all('span', class_='css-1jxf684 r-bcqeeo r-1ttztb7 r-qvutc0 r-poiln3')

        tweets = []
        for span in tweet_spans:
            tweet_text = span.get_text(strip=True)
            if tweet_text:
                tweets.append(tweet_text)

        return tweets

    def _extract_tweets(self, limit: int = None) -> TwitterContent:
        tweet_elements = self.driver.find_elements(By.XPATH, "//div[@data-testid='tweet']")

        if limit:
            tweet_elements = tweet_elements[:limit]

        tweets = []
        for tweet in tweet_elements:
            try:
                text_element = tweet.find_element(By.XPATH, ".//div[@data-testid='tweetText']")
                time_element = tweet.find_element(By.XPATH, ".//time")
                tweets.append(Tweet(
                    text=text_element.text,
                    timestamp=time_element.get_attribute('datetime')
                ))
            except Exception as e:
                logger.error(f"Error extracting tweet: {e}")

        return TwitterContent(
            tweets=tweets,
            page_source=self.driver.page_source
        )

    def cleanup(self):
        if self.driver:
            self.driver.quit()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.cleanup()

def get_tweets(num_tweets=5):
    with TwitterScraper() as scraper:
        scraper.login_to_twitter("skarandom","SK99@pass")
        return [x.text for x in scraper.get_home_page_content(num_tweets).tweets]

# Usage example
if __name__ == "__main__":
    print(get_tweets(2))


# import json
# import os
# import logging
# import networkx as nx
# import matplotlib.pyplot as plt
#
# from datetime import datetime
# from typing import List
# from .researcher import AiResearcher
# from .semantic_chunker import SemanticSimilarityTextChunker
# from .llm_service.base import LLMPrompt
# from .prompts import Prompt
#
# class MultiAgentQueryOrchestrator:
#     def __init__(self,response_level="medium",device = "Local",model = None):
#         self.device = device
#         self.results = {}
#         self.response_level = response_level
#         self.results_folder = False
#         self.researcher = AiResearcher()
#         self.setup_logging()
#
#         self.num_queries = 3
#         self.model = model
#         self.subquery_max_urls = 3
#         self.subquery_max_articles = 3
#         self.top_chunks_in_article = 3
#         self.max_subquery_relevant_chunks = 5
#         self.sentence_chunk_size_in_article = 3
#         self.setup_args()
#
#     def setup_logging(self):
#         logging.basicConfig(level=logging.INFO,
#                             format='%(asctime)s - %(levelname)s - %(message)s',
#                             handlers=[logging.StreamHandler()])
#         self.logger = logging.getLogger(__name__)
#
#     def create_results_folder(self):
#         timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
#         folder_name = f"Results/results_{timestamp}"
#         os.makedirs(folder_name, exist_ok=True)
#         return folder_name
#
#     def save_to_results(self, filename, content):
#         if self.results_folder:
#             full_path = os.path.join(self.results_folder, filename)
#             with open(full_path, 'w') as f:
#                 if isinstance(content, str):
#                     f.write(content)
#                 else:
#                     json.dump(content, f, indent=2)
#         else:
#             file_name= filename.rsplit('.', 1)[0]
#             self.results.update({file_name:content})
#         self.logger.info(f"Saved {filename} to results folder")
#
#     def decompose_query(self, query: str, num_queries: int) -> List[str]:
#         self.logger.info(f"Decomposing query: {query}")
#         decompose_prompt = LLMPrompt(Prompt.DECOMPOSE_PROMPT,model=self.model,
#                                      device=self.device
#                                      )
#         subqueries_str = decompose_prompt.run(websearchquery=query,
#                                               num_queries=num_queries,
#                                               return_type=list)
#         subqueries_str = subqueries_str.replace("'", '"')
#         subqueries = json.loads(subqueries_str)
#         self.save_to_results("subqueries.json", subqueries)
#         return subqueries
#
#     def visualize_dependency_graph(self, G: nx.DiGraph):
#         self.logger.info("Visualizing dependency graph")
#         plt.figure(figsize=(12, 8))
#         pos = nx.spring_layout(G)
#         nx.draw(G, pos, with_labels=True, node_color='lightblue',
#                 node_size=2000, font_size=10, arrows=True)
#
#         labels = nx.get_node_attributes(G, 'query')
#         label_pos = {k: (v[0], v[1] + 0.02) for k, v in pos.items()}
#         nx.draw_networkx_labels(G, label_pos, labels, font_size=8, font_weight="bold")
#
#         edge_labels = {(u, v): "" for (u, v) in G.edges()}
#         nx.draw_networkx_edge_labels(G, pos, edge_labels=edge_labels, font_size=8)
#
#         plt.title("Subquery Dependency Graph")
#         plt.axis('off')
#         plt.tight_layout()
#         filename = os.path.join(self.results_folder, "dependency_graph.png")
#         plt.savefig(filename, format="png", dpi=300, bbox_inches="tight")
#         plt.close()
#         self.logger.info(f"Graph visualization saved as {filename}")
#
#     def create_dependency_graph(self, subqueries: List[str]) -> nx.DiGraph:
#         self.logger.info("Creating dependency graph")
#         G = nx.DiGraph()
#         for i, sq in enumerate(subqueries):
#             G.add_node(i, query=sq)
#
#         for i in range(len(subqueries)):
#             for j in range(i + 1, len(subqueries)):
#                 dependency_prompt = LLMPrompt(Prompt.NODE_DEPENDENCY_PROMPT,model=self.model,
#                                               device=self.device
#                                               )
#                 dependency = dependency_prompt.run(first=subqueries[i],
#                                                    second=subqueries[j]).strip().lower()
#                 if 'yes' in dependency:
#                     G.add_edge(j, i)
#
#         return G
#
#     def process_subquery(self, subquery: str, context: str = "") -> str:
#         self.logger.info(f"Processing subquery: {subquery}")
#         articles = self.researcher.get_query_content(subquery, max_articles=self.subquery_max_articles,
#                                                      max_urls=self.subquery_max_urls)
#         chunker = SemanticSimilarityTextChunker(subquery)
#         all_chunks = []
#         for article in articles:
#             chunks = chunker.chunk_text(article,
#                                         top_chunks_in_article=self.top_chunks_in_article,
#                                         chunk_size=self.sentence_chunk_size_in_article)
#             all_chunks.extend(chunks)
#         all_chunks.sort(key=lambda chunk: chunker.score_sentence(chunk, chunker.get_query_keywords()), reverse=True)
#         relevant_chunks = all_chunks[:self.max_subquery_relevant_chunks]  # Top 3 chunks
#
#         combined_input = context + "\n\n" + "\n".join(relevant_chunks)
#         answer_prompt = LLMPrompt(Prompt.ANSWER_PROMPT,model=self.model,
#                                   device=self.device
#                                   )
#         return answer_prompt.run(combined_input=combined_input, subquery=subquery)
#
#     def save_graph(self, G: nx.DiGraph):
#         self.logger.info("Saving dependency graph")
#         data = nx.node_link_data(G)
#         self.save_to_results("dependency_graph.json", data)
#
#     def load_graph(self) -> nx.DiGraph:
#         self.logger.info("Loading dependency graph")
#         filename = os.path.join(self.results_folder, "dependency_graph.json")
#         with open(filename, 'r') as f:
#             data = json.load(f)
#         return nx.node_link_graph(data)
#
#     def extract_relevant_info(self, subquery: str, web_result: str, child_context: str) -> str:
#         self.logger.info(f"Extracting relevant info for subquery: {subquery}")
#         runner = LLMPrompt(Prompt.RELEVANT_INFO_PROMPT,model=self.model,
#                            device=self.device
#                            )
#         return runner.run(
#             subquery=subquery,
#             web_result=web_result,
#             child_context=child_context
#         )
#
#     def process_query(self, query: str, num_queries=None) -> str:
#         if num_queries is None:
#             num_queries = self.num_queries
#         self.logger.info(f"Processing main query: {query}")
#         subqueries = self.decompose_query(query=query, num_queries=num_queries)
#         self.logger.info(f"Subqueries\n {subqueries}")
#         graph = self.create_dependency_graph(subqueries)
#         if self.results_folder:
#             self.visualize_dependency_graph(graph)
#             self.save_graph(graph)
#
#         results = {}
#         for node in reversed(list(nx.topological_sort(graph))):
#             results[node] = {}
#             subquery = graph.nodes[node]['query']
#             self.logger.info(f"Processing subquery: {subquery}")
#             web_result = self.process_subquery(subquery)
#             results[node]["web_result"] = web_result
#
#             child_results = [results.get(child, {}).get("relevant_info", "") for child in graph.successors(node)]
#             child_context = "\n".join(child_results)
#
#             relevant_info = self.extract_relevant_info(subquery, web_result, child_context)
#             results[node]["relevant_info"] = relevant_info
#
#         self.save_to_results("subquery_results.json", results)
#
#         final_context = "\n".join([f"Subquery {i + 1}: {subquery}\nResult: {results[i]['relevant_info']}\n"
#                                    for i, subquery in enumerate(subqueries)])
#
#         self.logger.info("Generating final answer")
#         runner = LLMPrompt(Prompt.COMPREHENSIVE_ANSWER_PROMPT,model=self.model,
#                            device=self.device
#                            )
#         final_answer = runner.run(
#             final_context=final_context,
#             query=query
#         )
#         self.save_to_results("final_answer.txt", final_answer)
#         return final_answer
#
#     def setup_args(self):
#         if self.results_folder:
#             self.results_folder = self.create_results_folder()
#
#         if self.response_level=="low":
#             self.num_queries=3
#             self.model = "qwen2:1.5b-instruct" if self.model is None else self.model
#             self.subquery_max_urls = 2
#             self.subquery_max_articles = 2
#             self.top_chunks_in_article = 2
#             self.max_subquery_relevant_chunks = 2
#             self.sentence_chunk_size_in_article = 2
#         elif self.response_level=="high":
#             self.num_queries=8
#             self.model = "qwen2:7b-instruct" if self.model is None else self.model
#             self.subquery_max_urls = 4
#             self.subquery_max_articles = 4
#             self.top_chunks_in_article = 4
#             self.max_subquery_relevant_chunks = 5
#             self.sentence_chunk_size_in_article = 3
#         else:
#             self.num_queries=5
#             self.model = "qwen2:7b-instruct" if self.model is None else self.model
#             self.subquery_max_urls = 3
#             self.subquery_max_articles = 3
#             self.top_chunks_in_article = 3
#             self.max_subquery_relevant_chunks = 5
#             self.sentence_chunk_size_in_article = 3
#
#
# if __name__ == '__main__':
#     processor = MultiAgentQueryOrchestrator(
#             response_level="low",
#             device="GPU"
#         )
#     query = "when is pawan kalyan new movie OG release date?"
#     result = processor.process_query(query)
#     print(processor.results)


from typing import List

from langchain_text_splitters import RecursiveCharacterTextSplitter


class TextProcessor:
    @staticmethod
    def tokenize_text(text, chunk_size=100, chunk_overlap=20):
        text_splitter = RecursiveCharacterTextSplitter.from_tiktoken_encoder(
            chunk_size=chunk_size,
            chunk_overlap=chunk_overlap,
            separators=["\n\n", "\n", ".", "!", "?", " ", ""]
        )
        return text_splitter.split_text(text)

    @staticmethod
    def tokenize_list(content: List | None = None,
                       chunk_size: int = 100,
                       chunk_overlap: int = 20
                       ):
        contents = [x for x in content if x]
        splits = []
        for x in contents:
            splits.extend(TextProcessor.tokenize_text(x,chunk_overlap=chunk_overlap,chunk_size=chunk_size))
        return splits





from langchain_core.language_models import BaseChatModel

DESCRIPTOR_PROMPT : str = """
You are QueryRefiner, an AI assistant designed to clarify and refine user queries.
Your task is to take a user's informal or incomplete query and transform it into a clear, specific, and actionable request. Users often express their needs casually or incompletely due to familiarity or laziness. Your job is to interpret their intent and flesh out the details they may have omitted.

Given the user's QUERY:
<query_start>{query}</query_end>

Please refine it by:
1. Identifying the core request or question
2. Adding any missing context that seems implied
3. Specifying any vague terms or concepts
4. Structuring the query in a clear, complete sentence or question
5. Ensuring all necessary details are included for a database query or information retrieval task

Return the refined query in a format that would be clear and specific for an LLM to process.

Now, please refine the given query:
directly start giving the query, don't start like This is , Here is ... start now
           """

class QueryRefiner:
    @classmethod
    def refine_query(self,llm:BaseChatModel, query: str) -> str:
        return llm.invoke(DESCRIPTOR_PROMPT.format(query=query)).content





import json

from pydantic import BaseModel
from wordllama import WordLlama
from ..researcher_ai import RealTimeGoogleSearchProvider, UrlTextParser
from .text_splitter import TextProcessor


class SearchRetrieverResult(BaseModel):
    topk_chunks: list = []
    urls: list = []
    all_contents: list = []
    all_splits: list = []


class SearchRetriever:
    def __init__(self,chunk_overlap = 25,
chunk_size = 250,
max_urls = 5):
        self.chunk_overlap = chunk_overlap
        self.chunk_size = chunk_size
        self.max_urls = max_urls
        self.parser = UrlTextParser()
        self.searcher = RealTimeGoogleSearchProvider()
        self.splitter = WordLlama.load()

    def fetch_and_store_search_results(self, query, file_name_json):
        urls = self.searcher.perform_search(query,max_urls=self.max_urls)
        contents = self.parser.parse_html(urls)
        with open(file_name_json, "w", encoding='utf-8') as f:
            json.dump(contents, f, indent=2)

    def query_based_content_retrieval(self, query, topk=10, return_urls=False,
                                      verbose = False):
        urls = self.searcher.perform_search(query,max_urls=self.max_urls)
        if verbose:
            print(f"URLs found: {urls}")
        contents = self.parser.parse_html(urls)
        if verbose:
            print(f"len of contents: {len(contents)}")

        splits = TextProcessor.tokenize_list(contents,chunk_size=self.chunk_size,
                                             chunk_overlap=self.chunk_overlap)
        if verbose:
            print(f"len of splits: {len(splits)}")
        tokens = self.splitter.topk(query, splits, min(topk, len(splits)-1)) if len(splits)>2 else []
        if return_urls:
            return tokens, urls
        return SearchRetrieverResult(
            topk_chunks=tokens,
            urls=urls,
            all_contents=contents,
            all_splits = splits
        )


# Example usage
if __name__ == "__main__":
    retriever = SearchRetriever()

    # Fetch and store search results
    retriever.fetch_and_store_search_results("Python programming", "search_results.json")

    # Query-based content retrieval
    results = retriever.query_based_content_retrieval("Object-oriented programming in Python", topk=5)
    print(results)


from typing import List
from fastapi import FastAPI
from pydantic import BaseModel
from starlette.responses import StreamingResponse, JSONResponse
from langchain.chat_models import ChatOllama
from langchain.schema import HumanMessage

app = FastAPI()

class ChatInput(BaseModel):
    text: str
    model: str
    temperature: float
    num_predict: int

class BatchChatInput(BaseModel):
    messages: List[str]
    model: str
    temperature: float

# Store the current model instance
current_model = None
current_model_name = None

async def get_chat_model(model: str, temp: float, num_predict: int):
    global current_model, current_model_name

    # If the requested model is different from the current one, create a new instance
    if model != current_model_name:
        # Delete the previous model instance
        if current_model:
            del current_model


        # Create a new model instance
        current_model = ChatOllama(model=model, temperature=temp, num_predict=num_predict)
        current_model_name = model

    return current_model

async def chat_stream(text: str, model: str, temp: float, num_predict: int):
    chat_model = await get_chat_model(model, temp, num_predict)
    async for chunk in chat_model.astream([HumanMessage(content=text)]):
        yield chunk.content

async def process_batch(messages: List[str], model: str, temp: float):
    chat_model = await get_chat_model(model, temp, 0)  # Use 0 for num_predict in batch mode
    human_messages = [HumanMessage(content=msg) for msg in messages]
    responses = await chat_model.abatch(human_messages)
    return [response.content for response in responses]

@app.post("/chat")
async def chat_endpoint(chat_input: ChatInput):
    return StreamingResponse(chat_stream(chat_input.text, chat_input.model, chat_input.temperature, chat_input.num_predict), media_type="text/event-stream")

@app.post("/batch-chat")
async def batch_chat_endpoint(batch_input: BatchChatInput):
    results = await process_batch(batch_input.messages, batch_input.model, batch_input.temperature)
    return JSONResponse(content={"results": results})

if __name__ == "__main__":
    import uvicorn
    uvicorn.run(app, host="0.0.0.0", port=8888)



from langchain_core.language_models import BaseChatModel
from langchain_core.output_parsers import JsonOutputParser


class SearchQueryToNSubquery:
    @classmethod
    def ai_splits(self,llm: BaseChatModel,query: str) -> dict:
        messages = [
            {"role": "system",
             "content": "You are expert at understanding user query and generating list of google search queries rephrase and rewritten for better search results ,"
                        "that completely solves the given user query."
                        "Refine and expand the query , then generate the variations of the query"
                        "Output JSON format: {\"refined_splits\":[<list_of_better_google_search_queries>]}"},
            {"role": "user", "content": f"Query: {query}"}
        ]
        results = llm.invoke(messages).content

        try:
            output_parser = JsonOutputParser()
            parsed_results = output_parser.parse(results)
        except:
            return {'refined_splits':[]}
        return parsed_results


import logging

from .parse_url.html_parser import UrlTextParser
from .search_provider.searcher import RealTimeGoogleSearchProvider

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

class AiResearcher:
    def __init__(self, search_provider = 'google'):
        self.searcher = RealTimeGoogleSearchProvider(search_provider=search_provider)
        self.parser = UrlTextParser()

    def get_query_content(self,query,max_articles = 3,max_urls=6,return_urls=False):
        """
        Gets the top 3 URLs for the given query, fetches their content,
        returns the first three contents.'
        """
        urls = self.searcher.perform_search(query,
                                       max_urls=max_urls)
        urls = list(set(urls))
        logger.info(f"URls found: {urls}")
        content = self.parser.parse_html(urls=urls)
        content = [x for x in content if x]
        if return_urls:
            return content[:max_articles],urls
        return content[:max_articles]